/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jbpm.web;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jbpm.JbpmConfiguration;
import org.jbpm.JbpmContext;
import org.jbpm.msg.command.CommandExecutorThread;
import org.jbpm.scheduler.impl.SchedulerThread;

/**
 * the jBPM servlet.
 */
 public class JbpmThreadsServlet extends HttpServlet {

  private static final long serialVersionUID = 1L;

  String jbpmConfigurationResource = null;
  String jbpmContextName = null;

  JbpmConfiguration jbpmConfiguration = null;
  CommandExecutorThread commandExecutorThread = null;
  SchedulerThread schedulerThread = null;

  public void init() throws ServletException {
    // get the jbpm configuration resource
    this.jbpmConfigurationResource = getInitParameter("jbpm.configuration.resource");
    
    if (jbpmConfigurationResource==null) {
      log.debug("using default jbpm cfg resource");
    } else {
      log.debug("using jbpm cfg resource: '"+jbpmConfigurationResource+"'");
    }

    // get the jbpm context to be used from the jbpm configuration
    this.jbpmContextName = getInitParameter("jbpm.context.name");

    if (jbpmContextName==null) {
      log.debug("using default jbpm context");
      jbpmContextName = JbpmContext.DEFAULT_JBPM_CONTEXT_NAME;
    } else {
      log.debug("using jbpm context: '"+jbpmContextName+"'");
    }

    JbpmConfiguration jbpmConfiguration = JbpmConfiguration.getInstance(jbpmConfigurationResource);

    commandExecutorThread = new CommandExecutorThread(jbpmConfiguration);
    commandExecutorThread.start();

    schedulerThread = new SchedulerThread(jbpmConfiguration);
    schedulerThread.start();
  }
  
  protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
    PrintWriter out = response.getWriter();
    out.println("<html>");
    out.println("<body>");
    out.println("<h2>JBoss jBPM Threads Servlet</h2><hr />");
    out.println("</body>");
    out.println("</html>");
  }
  
  private static Log log = LogFactory.getLog(JbpmThreadsServlet.class);
}
