/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jbpm.msg.db;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jbpm.JbpmConfiguration;

public abstract class StaticNotifier {

  private static final long serialVersionUID = 1L;

  static Map notificationObjects = Collections.synchronizedMap(new HashMap());
  
  public static void notify(String destination) {
    Object notificationObject = getNotificationObject(destination);
    synchronized (notificationObject) {
      log.trace("notifying one thread ("+destination+", "+notificationObject+")");
      notificationObject.notify();
    }
  }

  public static void waitForNotification(String destination) throws InterruptedException {
    try {
      Object notificationObject = getNotificationObject(destination);
      synchronized (notificationObject) {
        log.trace("going to wait for ("+destination+", "+notificationObject+")");
        long timeout = JbpmConfiguration.Configs.getLong("jbpm.msg.wait.timout");
        notificationObject.wait(timeout);
      }
      log.trace("woke up");
    } catch (InterruptedException e) {
      log.trace("waiting for notification got interrupted");
      throw e;
    }
  }
  
  static synchronized Object getNotificationObject(String destination) {
    Object notificationObject = notificationObjects.get(destination);
    if (notificationObject==null ) {
      notificationObject = new Object();
      notificationObjects.put(destination, notificationObject);
    }
    return notificationObject;
  }
  
  private static Log log = LogFactory.getLog(StaticNotifier.class);
}
